/*
 *  TOPPERS/ASP Kernel
 *      Toyohashi Open Platform for Embedded Real-Time Systems/
 *      Advanced Standard Profile Kernel
 * 
 *  Copyright (C) 2000-2003 by Embedded and Real-Time Systems Laboratory
 *                              Toyohashi Univ. of Technology, JAPAN
 *  Copyright (C) 2005-2015 by Embedded and Real-Time Systems Laboratory
 *              Graduate School of Information Science, Nagoya Univ., JAPAN
 * 
 *  上記著作権者は，以下の(1)〜(4)の条件を満たす場合に限り，本ソフトウェ
 *  ア（本ソフトウェアを改変したものを含む．以下同じ）を使用・複製・改
 *  変・再配布（以下，利用と呼ぶ）することを無償で許諾する．
 *  (1) 本ソフトウェアをソースコードの形で利用する場合には，上記の著作
 *      権表示，この利用条件および下記の無保証規定が，そのままの形でソー
 *      スコード中に含まれていること．
 *  (2) 本ソフトウェアを，ライブラリ形式など，他のソフトウェア開発に使
 *      用できる形で再配布する場合には，再配布に伴うドキュメント（利用
 *      者マニュアルなど）に，上記の著作権表示，この利用条件および下記
 *      の無保証規定を掲載すること．
 *  (3) 本ソフトウェアを，機器に組み込むなど，他のソフトウェア開発に使
 *      用できない形で再配布する場合には，次のいずれかの条件を満たすこ
 *      と．
 *    (a) 再配布に伴うドキュメント（利用者マニュアルなど）に，上記の著
 *        作権表示，この利用条件および下記の無保証規定を掲載すること．
 *    (b) 再配布の形態を，別に定める方法によって，TOPPERSプロジェクトに
 *        報告すること．
 *  (4) 本ソフトウェアの利用により直接的または間接的に生じるいかなる損
 *      害からも，上記著作権者およびTOPPERSプロジェクトを免責すること．
 *      また，本ソフトウェアのユーザまたはエンドユーザからのいかなる理
 *      由に基づく請求からも，上記著作権者およびTOPPERSプロジェクトを
 *      免責すること．
 * 
 *  本ソフトウェアは，無保証で提供されているものである．上記著作権者お
 *  よびTOPPERSプロジェクトは，本ソフトウェアに関して，特定の使用目的
 *  に対する適合性も含めて，いかなる保証も行わない．また，本ソフトウェ
 *  アの利用により直接的または間接的に生じたいかなる損害に関しても，そ
 *  の責任を負わない．
 * 
 *  @(#) $Id: target_timer.h 301 2015-01-07 04:57:01Z ertl-ishikawa $
 */

/*
 *  タイマドライバ（BlueNinja用）
 */

#ifndef TOPPERS_TARGET_TIMER_H
#define TOPPERS_TARGET_TIMER_H

#include "tz1000.h"

/*
 *  タイマ割込みハンドラ登録のための定数
 */
#define INTPRI_TIMER    (-5)          /* 割込み優先度 */
#define INTATR_TIMER    0U            /* 割込み属性 */
#define INHNO_TIMER     INTNO_TMR1 /* 割込みハンドラ番号 */
#define INTNO_TIMER     INTNO_TMR1 /* 割込み番号 */

/*
 *  タイマ値の内部表現の型
 */
typedef uint32_t    CLOCK;

/*
 * =begin not implemented...
 * TODO: オーバランタイマは現状未サポート
 */
#ifdef TOPPERS_SUPPORT_OVRHDR
/*
 *  GPTM0_Aを使用する
 */

/*
 *	タイマ割込みハンドラ登録のための定数
 */
#define INHNO_OVRTIMER 	XX   /* 割込みハンドラ番号 */
#define INTNO_OVRTIMER 	XX   /* 割込み番号 */
#define INTPRI_OVRTIMER	(-6)		  /* 割込み優先度 */
#define INTATR_OVRTIMER	0U			  /* 割込み属性 */

#endif /* TOPPERS_SUPPORT_OVRHDR */

#ifndef TOPPERS_MACRO_ONLY
#ifdef TOPPERS_SUPPORT_OVRHDR
/*
 *  タイマ値の内部表現とマイクロ秒単位との変換
 */
#define OVRTIMER_CLOCK_UNIT 50
/*
 *  設定できる最大のタイマ周期（単位は内部表現）
 */
#define MAX_OVRTIMER_CLOCK    ((CLOCK) 0xffffffffU)
/*
 *	タイマ値の内部表現とマイクロ秒単位との変換
 *	1μsecあたりのクロックカウント数
 */
#define OVRTIMER_CLOCK(usec) ((usec) * OVRTIMER_CLOCK_UNIT)
#define OVRTIMER_USEC(clock) ((clock) / OVRTIMER_CLOCK_UNIT)

/*
 *	オーバランタイマの初期化処理
 *
 *	タイマを初期化する．
 */
extern void target_ovrtimer_initialize(intptr_t exinf);

/*
 *	オーバランタイマの終了処理
 *
 *	タイマの動作を終了する．
 */
extern void target_ovrtimer_terminate(intptr_t exinf);

/*
 *	オーバランタイマの起動処理
 *
 *	タイマを起動する．
 */
extern void target_ovrtimer_start(OVRTIM ovrtim);

/*
 *	オーバランタイマの停止処理
 *
 *	タイマの動作を停止させる．
 */
extern OVRTIM target_ovrtimer_stop(void);

/*
 *	オーバランタイマの現在値の読出し
 */
extern OVRTIM target_ovrtimer_get_current(void);

/*
 *	オーバランタイマ割込みハンドラ
 */
extern void target_ovrtimer_handler(void);

/*
 *  オーバランハンドラ起動ルーチン
 */
extern void	call_ovrhdr(void);
#endif /* TOPPERS_SUPPORT_OVRHDR */
/*
 * =end not implemented...
 */

/*
 *  高分解能タイマの現在のカウント値の読出し
 */
extern HRTCNT target_hrt_get_current(void);

/*
 *  高分解能タイマへの割込みタイミングの設定
 *
 *  高分解能タイマを，hrtcntで指定した値カウントアップしたら割込みを発
 *  生させるように設定する．
 */
extern void target_hrt_set_event(HRTCNT hrtcnt);

/*
 *  高分解能タイマ割込みの要求
 */
extern void target_hrt_raise_event(void);

/*
 *  タイマ割込みハンドラ
 */
extern void target_hrt_handler(void);

/*
 *  タイマの起動処理
 *
 *  タイマを初期化し，周期的なタイマ割込み要求を発生させる．
 */
extern void target_hrt_initialize(intptr_t exinf);

/*
 *  タイマの停止処理
 *
 *  タイマの動作を停止させる．
 */
extern void target_hrt_terminate(intptr_t exinf);

/*
 *  タイマ値の内部表現とマイクロ秒単位との変換
 *  48MHz / 2
 */
#define HRTCNT_CLOCK_UNIT 24
/*
 *  設定できる最大のタイマ周期（単位は内部表現）
 *  TCYC_HRTCNT(178956970us)*HRT_CLOCK_UNITがタイマ周期
 */
#define MAX_HRTCNT_CLOCK    ((HRTCNT)0xFFFFFFF0U)
/*
 *	タイマ値の内部表現とマイクロ秒単位との変換
 *	1μsecあたりのクロックカウント数
 */
#define HRTCNT_CLOCK(usec) ((usec) *  HRTCNT_CLOCK_UNIT)
#define HRTCNT_USEC(clock) ((clock) / HRTCNT_CLOCK_UNIT)

/*
 *  割込みタイミングに指定する最大値 [ASPD1005]
 *  タイマで計測できる最大の時間（μ秒）- タイマ割込みの最大遅延時間
 *  = (0xffffffff / 24) - MAX_HRTINT_DLY
 *  ~ 178956970 us - MAX_HRTINT_DLY
 */
#ifndef MAX_HRTINT_DLY
#define MAX_HRTINT_DLY 5U   /* デフォルトのタイマ割込みの遅延時間 */
#endif /* MAX_HRTINT_DLY */
#define HRTCNT_BOUND    (178956970U - MAX_HRTINT_DLY)

#endif /* TOPPERS_MACRO_ONLY */

#endif /* TOPPERS_TARGET_TIMER_H */
